@@ -61,10 +61,10 @@ typedef uchar bool;
 // between edbrowse and the js engine.
 typedef void *jsobjtype;
 
+extern int ignore; // used to suppress warnings
 extern const char *jsSourceFile; // sourcefile providing the javascript
 extern int jsLineno; // js line number
 extern int undo1line; // special undo line number
-extern int imapfetch; // fetch this many envelopes in imap
 extern char frameContent[];
 	extern const char startWindowJS[];
 	extern const char deminJS[];
@@ -138,7 +138,7 @@ typedef uchar *pst;		/* perl string */
 #define MAXPROXY 200 // number of proxy entries
 #define MAXDBT 100 // number of configured database tables
 #define MAXTCOLS 40 // columns in a configured table
-#define MAXSESSION 1000 // concurrent edbrowse sessions
+#define MAXSESSION 10000 // concurrent edbrowse sessions
 /* Allocation increment for a growing string, that we don't expect
  * to get too large.  This must be a power of 2. */
 #define ALLOC_GR        0x100
@@ -195,6 +195,7 @@ struct i_get {
 	char *referrer;
 	CURL *h;
 	struct ebFrame *cf; // current frame
+	const struct htmlTag *gotag; // <a> tag
 	int tsn; // thread sequence number
 // State of download to disk, see http.c for state values.
 	int down_state;
@@ -238,9 +239,16 @@ struct MACCOUNT { // email account, pop3 or imap
 	char *inurl, *outurl;
 	int inport, outport;
 	uchar inssl, outssl;
-	bool nofetch, imap, secure;
+	bool nofetch, imap, secure, maskon, maskactive, dxon, move_capable, mc_set;
 	const char *cclist[MAXCC + 1]; // extra cc directives
 	uchar cctype[MAXCC]; // cc or bcc or attach
+	const char *isub; // substring of folders
+	const char *masktext;
+	uchar maskfolder[256];
+	const char *dxtext;
+	uchar dxfolder[256];
+	uchar dxtrash;
+	char lastletter;
 };
 extern struct MACCOUNT accounts[];	/* all the email accounts */
 extern int maxAccount;		/* how many email accounts specified */
@@ -260,7 +268,7 @@ struct DBTABLE {
 	char *name, *shortname;
 	char *cols[MAXTCOLS];
 	int ncols;
-	unsigned char key1, key2, key3, key4;
+	uchar key1, key2, key3, key4;
 	char *types;
 	char *nullable;
 };
@@ -289,8 +297,8 @@ extern uchar browseLocal;
 extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
 extern bool curlActive; // is curl running?
 extern bool ismc;		// Is the program running as a mail client?
-extern bool isimap;		// Is the program running as an imap client?
 extern bool passMail;		// pass mail across the filters
+extern bool attimg; // show atached images
 extern bool down_bg;		// download in background
 extern bool down_jsbg;		// download javascript in background
 extern bool re_utf8, endMarks;
@@ -324,6 +332,7 @@ extern char *mailReply;		/* file to hold reply info for each email */
  * but you never know. Should probably live somewhere under .Trash */
 extern char *mailStash;
 extern char *downDir;		/* the download directory */
+extern char *down_prefile; // set file name ahead of time
 extern char *ebTempDir;		/* edbrowse temp, such as /tmp/.edbrowse */
 extern char *ebUserDir;		/* $ebTempDir/nnn user ID appended */
 extern char *dbarea, *dblogin, *dbpw;	/* to log into the database */
@@ -350,6 +359,7 @@ extern pst linePending;
 extern char *changeFileName;
 extern char *addressFile;	// your address book
 extern char *emojiFile;	// your library of emojis
+extern char *irclog; // log all your irc sessions
 extern int redirect_count;
 extern char *serverData;
 extern int serverDataLen;
@@ -361,6 +371,7 @@ extern char *cookieFile;	/* persistent cookies */
 extern char *cacheDir;	/* directory for a persistent cache of http pages */
 extern int cacheSize; // in megabytes
 extern int cacheCount; // number of cache files
+extern bool hlocal; // http local, read from cache only
 
 // General link list. This is, interestingly, the same design
 // that Fabrice came up with for his quickjs project.
@@ -400,6 +411,7 @@ struct ebFrame {
 	const char *charset;		// charset for this page
 	bool render1; // rendered via protocol or urlmatch
 	bool render2; // rendered via suffix
+	bool render3; // rendered before load, only once
 	bool render1b;
 	bool baseset; // <base> tag has been seen
 	bool uriEncoded; // filename is url encoded
@@ -447,12 +459,11 @@ struct ebWindow {
 	struct ebFrame *jdb_frame; // if in jdb mode
 	char *referrer; // another web page that brought this one to life
 	char *baseDirName;	// when scanning a directory
-	char *htmltitle, *htmldesc, *htmlkey;	// title, description, keywords
+	char *htmltitle, *htmlauthor, *htmldesc, *htmlgen, *htmlkey;	// title, author, description, generator, keywords
 	char *saveURL;		// for the fu command
 	char *mailInfo;
 	char lhs[MAXRE], rhs[MAXRE];	/* remembered substitution strings */
 	struct lineMap *map, *r_map;
-	struct listHead lines, r_lines;
 	char *dmap; // for directory listing
 /* The labels that you set with the k command, and access via 'x.
  * Basically, that's 26 line numbers.
@@ -496,15 +507,24 @@ int ehsn;
 	bool sqlMode:1;		// accessing a table
 	bool irciMode:1;		// input irc
 	bool ircoMode:1;		// output irc
+	bool ircoMode1:1;		// was output irc
 	bool ircChannels:1; // show channel when displaying messages
 	bool ircSecure:1; // ssl connection
 	bool ircPingOut:1; // sent ping to server
+	bool imapMode1:1;		// imap folders
+	bool imapMode2:1;		// imap envelopes
+	bool imapMode3:1;		// reading an email
 	int irc_fd; // file descriptor for socket
 	void *irc_ssl; // stream for a secure socket
 	ushort ircOther; // point to the output side
 	ushort ircCount;
 	char *ircNick, *ircChannel;
 	time_t ircRespond;
+	int imap_n; // imap account
+	char imap_env[8]; // envelope format
+	int imap_l; // fetch limit
+	CURL *imap_h; // imap server handle
+	char *mail_raw; // the email you're looking at
 	struct DBTABLE *table;	// if in sqlMode
 	time_t nextrender;
 };
@@ -599,6 +619,7 @@ struct htmlTag {
 	bool inxhr:1; // script is really an xhr
 	bool rdonly:1;
 	bool disabled:1;
+	bool hidden:1;
 	bool clickable:1;	/* but not an input field */
 	bool secure:1;
 	bool scriptgen:1; // script generated, not from source
