@@ -11,21 +11,28 @@ extern "C" {
 #endif
 
 // sourcefile=buffers.c
-void setHistcontrol(void);
 void undoSpecialClear(void);
-void removeHiddenNumbers(pst p, uchar terminate, int cx);
+void removeHiddenNumbers(pst p, uchar terminate, int cx, const Window *w);
+pst fetchLineWindow(int n, int show, const Window *w);
+pst fetchLineContext(int n, int show, int cx);
 pst fetchLine(int n, int show);
 void displayLine(int n);
 void printDot(void);
+void printPrompt(void);
 void initializeReadline(void);
-pst inputLine(void);
+pst inputLine(const bool textEntry);
+void saveSubstitutionStrings(void);
+void restoreSubstitutionStrings(Window *nw);
+Window *createWindow(void);
 void freeWindowLines(struct lineMap *map);
+void freeWindows(int cx, bool all);
 bool cxCompare(int cx) ;
 bool cxActive(int cx, bool error);
 bool cxQuit(int cx, int action) ;
 void cxSwitch(int cx, bool interactive) ;
 void addToMap(int nlines, int destl);
-bool addTextToBuffer(const pst inbuf, int length, int destl, bool showtrail) ;
+bool addTextToBuffer(const uchar *inbuf, int length, int destl, bool showtrail) ;
+void addTextToBackend(const char *inbuf);
 void delText(int start, int end) ;
 bool readFileArgv(const char *filename, int fromframe, const char *orig_head);
 bool writeFile(const char *name, int mode);
@@ -35,7 +42,7 @@ bool runCommand(const char *line);
 bool edbrowseCommand(const char *line, bool script) ;
 int sideBuffer(int cx, const char *text, int textlen, const char *bufname);
 void freeEmptySideBuffer(int n);
-bool browseCurrentBuffer(void) ;
+bool browseCurrentBuffer(const char *suffix, bool plain) ;
 bool locateTagInBuffer(int tagno, int *ln1_p, int *ln2_p, char **p1_p, char **p2_p, char **s_p, char **t_p);
 char *getFieldFromBuffer(int tagno, int ln0);
 int fieldIsChecked(int tagno);
@@ -59,14 +66,32 @@ bool ebConnect(void) ;
 int goSelect(int *startLine, char **rbuf) ;
 
 // sourcefile=fetchmail.c
-bool setEnvelopeFormat(const char *s);
+void setEnvelopeFormat(const char *s);
+void setFetchLimit(const char *s);
 int fetchMail(int account);
 int fetchAllMail(void);
-void scanMail(void);
+void scanUnreadMail(void);
 bool emailTest(void);
 void mail64Error(int err);
-char *emailParse(char *buf);
+char *emailParse(char *buf, bool plain);
 bool setupReply(bool all);
+bool imapBufferPresent(void);
+bool imapBuffer(char *line);
+bool imap1rf();
+bool folderDescend(const char *path, bool rf);
+bool folderSearch(const char *path, char *search, bool rf);
+bool mailDescend(const char *path, char cmd);
+bool imapMarkRead(int l1, int l2, char sign);
+bool imapMovecopy(int l1, int l2, char cmd, char *dest);
+bool imapDelete(int l1, int l2, char cmd);
+bool imapMovecopyWhileReading(char cmd, char *dest);
+bool imapDeleteWhileReading(void);
+bool saveEmailWhileReading(char key, const char *name);
+bool saveEmailWhileEnvelopes(char key, const char *name);
+bool rfWhileReading();
+bool addFolders();
+bool deleteFolder(int ln);
+bool renameFolder(const char *src, const char *dest);
 
 // sourcefile=format.c
 void prepareForBrowse(char *h, int h_len);
@@ -211,6 +236,7 @@ const char *findAgentForURL(const char *url);
 const char *mailRedirect(const char *to, const char *from, const char *reply, const char *subj);
 int runEbFunction(const char *line);
 const char *getInputLineFromScript(void);
+bool runBuffer(int b, const Window *w, bool stopflag, int ln1, int ln2);
 struct DBTABLE *findTableDescriptor(const char *sn);
 struct DBTABLE *newTableDescriptor(const char *name);
 bool readConfigFile(void);
@@ -318,7 +344,7 @@ char getLetter(const char *s) ;
 char *getFileName(int msg, const char *defname, bool isnew, bool ws);
 int shellProtectLength(const char *s);
 void shellProtect(char *t, const char *s);
-char *dirSuffixContext(int n, int cx);
+char *dirSuffixWindow(int n, const Window *w);
 char *dirSuffix(int n);
 char *dirSuffix2(int n, const char *path);
 char *makeAbsPath(const char *f);
@@ -340,9 +366,8 @@ void unpercentURL(char *url) ;
 void unpercentString(char *s) ;
 char *percentURL(const char *start, const char *end);
 bool looksPercented(const char *start, const char *end);
-char *htmlEscape0(const char *s, bool do_and);
-#define htmlEscape(s) htmlEscape0((s), true)
-#define htmlEscapeTextarea(s) htmlEscape0((s), false)
+char *htmlEscape0(const char *s, bool para);
+#define htmlEscape(s) htmlEscape0((s), false)
 bool isURL(const char *url) ;
 bool isSQL(const char *url);
 bool isBrowseableURL(const char *url) ;
@@ -374,15 +399,17 @@ void findcookies(char **s, int *l, const char *url, bool issecure) ;
 void mergeCookies(void);
 void setupEdbrowseCache(void);
 void clearCache(void) ;
-bool fetchCache(const char * url, const char *etag, time_t modtime, char **data, int *data_len) ;
-bool presentInCache(const char *url) ;
+bool fetchCache(const char * url, const char *etag, time_t modtime, bool grab, char **data, int *data_len) ;
+bool presentInCache(const char *url, bool *recent) ;
 void storeCache(const char *url, const char *etag, time_t modtime, const char *data, int datalen) ;
 bool getUserPass(const char *url, char *creds, bool find_proxy) ;
 bool getUserPassRealm(const char *url, char *creds, const char *realm);
 // Add authorization entries only in the foreground, but it's an
 // atomic operation, so the previous routines can run from other threads.
 bool addWebAuthorization(const char *url, const char *credentials, bool proxy, const char *realm);
+const char *edbrowseTempFilename(const char *suffix, bool output);
 const struct MIMETYPE *findMimeByURL(const char *url, uchar *sxfirst);
+const struct MIMETYPE *findMimeBySuffix(const char *suffix);
 const struct MIMETYPE *findMimeByFile(const char *filename);
 const struct MIMETYPE *findMimeByContent(const char *content);
 bool runPluginCommand(const struct MIMETYPE *m, const char *inurl, const char *infile, const char *indata, int inlength, char **outdata, int *outlength);
