@@ -51,7 +51,7 @@ void prepareForField(char *h)
 	while (*h) {
 // this line is goofy cause null would terminate the loop
 		if (*h == 0) *h = ' ';
-		if (*h == '\r') *h = '\07';
+		if (*h == '\n') *h = '\07'; // should never happen
 		if (*h == InternalCodeChar) *h = InternalCodeCharAlternate;
 		++h;
 	}
@@ -302,8 +302,10 @@ static void anchorSwap(char *buf)
 
 		for (s = buf; (c = *s); ++s) {
 			if (isspaceByte(c) || (c == '|' && !premode)) {
+#if 0
 				if (c == '\t' && !premode)
 					*s = ' ';
+#endif
 				if (!w)
 					w = s;
 				continue;
@@ -625,17 +627,16 @@ static void appendOneChar(char c)
 		*bl_cursor++ = c;
 }
 
-static bool spaceNotInInput(void)
+static bool spacesInInput(void)
 {
 	char *t = bl_cursor;
 	char c;
 	for (--t; t >= bl_start; --t) {
 		c = *t;
-		if (c == '\n' || c == '\r')
-			return true;
+		if (c == '\n') return false;
 		if (c == '>' && t >= bl_start + 2 &&
 		    t[-1] == '0' && t[-2] == InternalCodeChar)
-			return true;
+			return false;
 		if (c != '<')
 			continue;
 		while (t > bl_start && isdigitByte(t[-1]))
@@ -643,23 +644,40 @@ static bool spaceNotInInput(void)
 		if (*t == '<')
 			continue;
 		if (t > bl_start && t[-1] == InternalCodeChar)
-			return false;
+			return true;
 	}
-	return true;
+	return false;
 }
 
 static void appendSpaceChunk(const char *chunk, int len, bool premode)
 {
-	int nlc = pre_cr;	/* newline count */
-	int spc = 0;		/* space count */
+	int nlc = pre_cr;	// newline count
+	int spc = 0;		// space count
 	int i, j;
 	char c, d, e;
 
 	if (!len)
+		return; // nothing to add
+
+// spaces in input field are literal
+	if(spacesInInput()) {
+		for (i = 0; i < len; ++i) {
+			c = chunk[i];
+			if (c == '\n') // should never happen
+				c = '\r';
+			if (c == 0) // should never happen
+				c = ' ';
+			colno += (c == '\t' ? 4 : 1);
+			appendOneChar(c);
+		}
 		return;
+	}
+
+// gather stats on the whitespace chunk
 	for (i = 0; i < len; ++i) {
 		c = chunk[i];
-		if (c == '\n' || c == '\r') {
+// a newline or an isolated return
+		if (c == '\n' || (c == '\r' && (i == len || chunk[i+1] != '\n'))) {
 			++nlc, spc = 0;
 			continue;
 		}
@@ -670,7 +688,8 @@ static void appendSpaceChunk(const char *chunk, int len, bool premode)
 		++spc;
 	}
 
-	if (!premode && spaceNotInInput()) {
+	if (!premode) {
+// was there a period or such just before this whitespace?
 		int l = bl_cursor - bl_start;
 		c = d = ' ';
 		if (l)
@@ -682,7 +701,7 @@ static void appendSpaceChunk(const char *chunk, int len, bool premode)
 			e = c;
 		if (strchr(".?!:", e)) {
 			bool ok = true;
-/* Check for Mr. Mrs. and others. */
+// Check for Mr. Mrs. and others
 			if (e == '.' && bl_cursor - bl_start > 10) {
 				static const char *const prefix[] =
 				    { "mr.", "mrs.", "sis.", "ms.", 0 };
@@ -697,7 +716,7 @@ static void appendSpaceChunk(const char *chunk, int len, bool premode)
 				for (i = 0; prefix[i]; ++i)
 					if (strstr(trailing, prefix[i]))
 						ok = false;
-/* Check for John C. Calhoon */
+// Check for John C. Calhoon
 				if (isupperByte(bl_cursor[-2])
 				    && isspaceByte(bl_cursor[-3]))
 					ok = false;
@@ -714,14 +733,15 @@ static void appendSpaceChunk(const char *chunk, int len, bool premode)
 			lright = colno, idxright = l;
 		lany = colno, idxany = l;
 		if (formatOverflow) {
-/* tack a short fragment onto the previous line. */
+// tack a short fragment onto the previous line
 			if (longcut && colno <= 15 && (nlc || lperiod == colno)) {
 				bl_start[longcut] = ' ';
 				if (!nlc)
 					len = spc = 0, nlc = 1;
-			}	/* pasting small fragment onto previous line */
+			}	// pasting small fragment onto previous line
 		}
-	}			/* allowing line breaks */
+	}
+
 	if (lspace == 3)
 		nlc = 0;
 	if (nlc) {
@@ -1263,7 +1283,7 @@ static void cutDuplicateEmail(char *line, const char *dup, int duplen)
 	while (*line) {
 		s = strchr(line, ',');
 		if (!s)
-			return;	/* should never happen */
+			return;	// should never happen
 		if (duplen == s - line && memEqualCI(line, dup, duplen)) {
 			++s;
 			strmove(line, s);
@@ -1288,7 +1308,7 @@ void cutDuplicateEmails(char *tolist, char *cclist, const char *reply)
 	while (*s) {
 		t = strchr(s, ',');
 		if (!t)
-			break;	/* should never happen */
+			break;	// should never happen
 		len = t - s;
 		++t;
 		cutDuplicateEmail(t, s, len);
@@ -1300,15 +1320,15 @@ void cutDuplicateEmails(char *tolist, char *cclist, const char *reply)
 	while (*s) {
 		t = strchr(s, ',');
 		if (!t)
-			break;	/* should never happen */
+			break;	// should never happen
 		len = t - s;
 		++t;
 		cutDuplicateEmail(t, s, len);
 		s = t;
 	}
 
-/* If your email address is on the to or cc list, drop it.
- * But retain it if it is the reply, in case you sent mail to yourself. */
+// If your email address is on the to or cc list, drop it.
+// But retain it if it is the reply, in case you sent mail to yourself.
 	if (reply[0]) {
 		struct MACCOUNT *m = accounts;
 		int i;
@@ -1352,7 +1372,7 @@ bool isEmailAddress(const char *s)
 	return atfound & dotfound;
 }
 
-/* Check for a list of email addresses separated by commas. */
+// Check for a list of email addresses separated by commas.
 bool isEmailAddressList(char *s)
 {
 	if (!s || !*s)
@@ -1368,7 +1388,7 @@ bool isEmailAddressList(char *s)
 	return false;
 }
 
-/* return 1 for utf16, 2 for utf32, ored with 4 for big endian */
+// return 1 for utf16, 2 for utf32, ored with 4 for big endian
 int byteOrderMark(const uchar * buf, int buflen)
 {
 	if (buflen < 2)
@@ -1486,10 +1506,11 @@ Don't forget to free it when you're done.
 static const unsigned int iso_unicodes[2][128] = {
 	{
 /*********************************************************************
-The first 32 nonascii chars in iso8859-1 are control characters,
+The first 32 nonascii chars in iso8859-1 are symbols,
 and almost never used.
 Much more common are the cp1252 characters, introduced by Microsoft.
 I'm gonna go with those, and hope I'm right more often than wrong.
+There should be a switch for this.
 *********************************************************************/
 #define CP1252 1
 #if CP1252
@@ -2602,6 +2623,8 @@ static struct EJGROUP {
 	const char *name;
 	const char *desc[EJGROUPSIZE];
 	long code[EJGROUPSIZE];
+	long code2[EJGROUPSIZE];
+	char join[EJGROUPSIZE];
 } *ejgroup0;
 // the emojis file pulled into memory
 static char *ejbase;
@@ -2671,7 +2694,7 @@ void loadEmojis(void)
 			i_printf(MSG_EmojiBadControl, c, lineno);
 			goto fail;
 		}
-		if((signed char)c > ' ' && !isalnumByte(c) && c != '{' && c != '}') {
+		if((signed char)c > ' ' && !isalnumByte(c) &&  !strchr("{}+^", c)) {
 			i_printf(MSG_EmojiBadChar, c, lineno);
 			goto fail;
 		}
@@ -2683,7 +2706,8 @@ void loadEmojis(void)
 	s = ejbase, lineno = 0;
 	while(++lineno, *s) {
 		bool limitprint = false;
-		long uc;
+		long uc, uc2;
+		char joinchar;
 		t = strchr(s, '\n');
 		if(t) *t++ = 0; else t = s + strlen(s);
 		if(!*s) { // empty line
@@ -2719,6 +2743,11 @@ void loadEmojis(void)
 			i_printf(MSG_EmojiUnicode, lineno);
 			s = t; continue;
 		}
+		joinchar = 0;
+		if(*u == '^' || *u == '+') {
+			joinchar = *u;
+			uc2 = strtol(u + 1, &u, 16);
+		}
 		if(!*u) { // no description
 			s = t; continue;
 		}
@@ -2733,6 +2762,9 @@ void loadEmojis(void)
 		} else {
 			g->desc[g->n] = u+1;
 			g->code[g->n] = uc;
+			g->join[g->n] = joinchar;
+			if(joinchar)
+				g->code2[g->n] = uc2;
 			++g->n;
 		}
 		s = t;
@@ -2760,7 +2792,7 @@ static const struct EJGROUP *ejGroupSearch(const char *s, int len)
 	if(g1)
 		return g1;
 fail:
-	i_printf(g1 ? MSG_Many : MSG_NoMatch);
+	i_printf(g1 ? MSG_Multiple : MSG_NoMatch);
 	printf(": ");
 	while(len--)
 		printf("%c", *s++);
@@ -2920,7 +2952,7 @@ char *selectEmoji(const char *p, int len)
 		if(p[dot] == '.') break;
 	if(dot == len)
 		dot = 0;
-	if(dot == len - 1)
+	if(dot && dot == len - 1)
 		dot = 0, --len;
 
 	g = ejGroupSearch(p, (dot ? dot : len));
@@ -2930,8 +2962,12 @@ char *selectEmoji(const char *p, int len)
 	if(dot) { // foo.bar
 		n = ejSelect(g, p + dot + 1, len - dot - 1, false);
 		if(n >= 0) {
-			ejresponse = allocMem(7);
+			ejresponse = allocMem(15 + 1);
 			strcpy(ejresponse, uni2utf8(g->code[n]));
+			if(g->join[n] == '^')
+				strcat(ejresponse, uni2utf8(8205));
+			if(g->join[n])
+				strcat(ejresponse, uni2utf8(g->code2[n]));
 			return ejresponse;
 		}
 	}
@@ -2952,7 +2988,7 @@ top:
 		return 0;
 	}
 // picking one or more emojis
-	ejresponse = allocMem(6 * ejcommas + 6 + 1);
+	ejresponse = allocMem(15 * ejcommas + 15 + 1);
 	ejresponse[0] = 0;
 	s = ejline;
 	while(*s) {
@@ -2967,6 +3003,10 @@ top:
 			goto top;
 		}
 		strcat(ejresponse, uni2utf8(g->code[n]));
+			if(g->join[n] == '^')
+				strcat(ejresponse, uni2utf8(8205));
+			if(g->join[n])
+				strcat(ejresponse, uni2utf8(g->code2[n]));
 		s = t;
 	}
 // all good
@@ -3012,15 +3052,22 @@ We roll our own international messages in this file, so you wouldn't think
 we need setlocale, but pcre needs the locale for expressions like \w,
 and for ranges like [A-z],
 and to convert to upper or lower case etc.
-So I set LC_ALL, which covers both LC_CTYPE and LC_COLLATE.
-By calling strcoll, the directory scan is in the same order as ls.
-See dircmp_alph() in stringfile.c
-*********************************************************************/
-	setlocale(LC_ALL, "");
+I think we want a standard behavior here, so I'm going with C.
 
-/* But LC_TIME controls time/date formatting, I.E., strftime.  The one
- * place we do that, we need standard day/month abbreviations, not
- * localized ones.  So LC_TIME needs to be C. */
+By calling strcoll, the directory scan is in the same order as ls.
+But only if we get our collation information from the environment, as ls does.
+
+quickjs parses numbers according to locale, and that has to be
+standard C numbers, or all the javascript on the internet won't run.
+
+LC_TIME controls time/date formatting, I.E., strftime.  The place we do that,
+sendmail.c, we need standard day/month abbreviations, not  localized ones.
+According to the rfc standards. So LC_TIME needs to be C.
+*********************************************************************/
+
+	setlocale(LC_CTYPE, "C");
+	setlocale(LC_COLLATE, "");
+	setlocale(LC_NUMERIC, "C");
 	setlocale(LC_TIME, "C");
 
 	strncpy(eb_language, s, 7);
@@ -3244,7 +3291,7 @@ bool helpUtility(void)
 	undoSpecialClear();
 
 // maybe we already have a buffer with the help guide in it
-	for (cx = 1; cx < MAXSESSION; ++cx) {
+	for (cx = 1; cx <= maxSession; ++cx) {
 		Window *w = sessionList[cx].lw;
 		if (!w)
 			continue;
@@ -3262,7 +3309,7 @@ bool helpUtility(void)
 		return false;
 	cxSwitch(cx, false);
 	i_printf(MSG_MovedSession, cx);
-	browseCurrentBuffer();
+	browseCurrentBuffer(NULL, false);
 	cw->dot = 1;
 	return true;
 }
