@@ -13,7 +13,7 @@ CURL *global_http_handle;
 CURLSH *global_share_handle;
 bool pluginsOn = true;
 bool down_bg;			// download in background
-bool down_jsbg = true;		// download js in background
+bool down_jsbg;		// download js in background
 char showProgress = 'd';	// dots
 static char *httpLanguage;	/* outgoing */
 
@@ -823,6 +823,7 @@ bool httpConnect(struct i_get *g)
 	bool proceed_unauthenticated = false;
 	bool post_request = false;
 	bool head_request = false;
+	bool recent, local2 = hlocal;
 	uchar sxfirst = 0;
 	int n;
 
@@ -1076,7 +1077,7 @@ mimestream:
 
 	still_fetching = true;
 
-	if (!post_request && (g->headrequest || presentInCache(g->urlcopy))) {
+	if (!post_request && (g->headrequest || presentInCache(g->urlcopy, &recent))) {
 		head_request = true;
 		curl_easy_setopt(h, CURLOPT_NOBODY, 1l);
 	}
@@ -1103,7 +1104,14 @@ mimestream:
 
 perform:
 		g->is_http = g->cacheable = true;
-		curlret = fetch_internet(g);
+		if(local2) {
+			g->buffer = initString(&g->length);
+			g->headers = initString(&g->headers_len);
+			g->code = 200;
+			curlret = (head_request ? CURLE_OK : CURLE_COULDNT_CONNECT);
+		} else {
+			curlret = fetch_internet(g);
+		}
 
 /*********************************************************************
 This is a one line workaround for an apparent bug in curl.
@@ -1248,7 +1256,8 @@ they go where they go, so this doesn't come up very often.
 			goto perform;
 		}
 // get http code
-		curl_easy_getinfo(h, CURLINFO_RESPONSE_CODE, &g->code);
+		if(!local2)
+			curl_easy_getinfo(h, CURLINFO_RESPONSE_CODE, &g->code);
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 
@@ -1315,7 +1324,7 @@ they go where they go, so this doesn't come up very often.
 				if (curlret != CURLE_OK)
 					goto curl_fail;
 
-				if (!post_request && (g->headrequest || presentInCache(g->urlcopy))) {
+				if (!post_request && (g->headrequest || presentInCache(g->urlcopy, &recent))) {
 					head_request = true;
 					curl_easy_setopt(h, CURLOPT_NOBODY, 1l);
 				}
@@ -1384,7 +1393,7 @@ they go where they go, so this doesn't come up very often.
 					break;
 				}
 				if (fetchCache
-				    (g->urlcopy, g->etag, g->modtime,
+				    (g->urlcopy, g->etag, g->modtime, local2,
 				     &cacheData, &cacheDataLen)) {
 					nzFree(g->buffer);
 					g->buffer = cacheData;
@@ -1431,9 +1440,11 @@ curl_fail:
 		return false;
 	}
 
-	if ((g->code != 200 && g->code != 201 &&
-	     (g->foreground || debugLevel >= 2)) ||
-	    (g->code == 201 && debugLevel >= 3))
+// 200 ok 201 created 202 accepted 203 response may have been changed in some way 204 no content nostack
+// This is just printing the warning / error.
+// code outside the "ok" range, and foreground or db3
+	if ((g->code < 200 || g->code > 204 || g->code == 203) &&
+	     (g->foreground || debugLevel >= 3))
 		i_printf(MSG_HTTPError,
 			 g->code, message_for_response_code(g->code));
 
@@ -1857,7 +1868,7 @@ void ebcurl_setError(CURLcode curlret, const char *url, int action,
 	void (*fn) (int, ...);
 
 	if (!getProtHostURL(url, prot, host)) {
-/* this should never happen */
+// this should never happen
 		prot[0] = host[0] = 0;
 	}
 
@@ -2311,6 +2322,28 @@ gopher_transfer_fail:
 	return true;
 }
 
+/*********************************************************************
+Get a suggested download filename from the <a> tag.
+The result is allocated, and cleaned up for security.
+It's fair to ask whether the download protocol should call this function,
+or whether it is more general - whether it should be called from readFile()
+after httpConnect();
+Also, we can't defer to javascript, because we're in a new window.
+If js called setAttribute("download") I'm not sure what we do.
+*********************************************************************/
+
+static char *suggestedDownloadName(const Tag *t)
+{
+	char *e, *f;
+	const char *e0;
+	if(!t) return 0;
+	e0 = attribVal(t, "download");
+	if(e0 && *e0) e = cloneString(e0); else return 0;
+	for(f = e; *f; ++f)
+		if(*f == '/' || *f == '\\') *f = '_';
+	return e;
+}
+
 static bool dataConnect(struct i_get *g)
 {
 	char *copy = cloneString(g->url);
@@ -2320,7 +2353,7 @@ static bool dataConnect(struct i_get *g)
 	if(!comma) { // should never happen
 		debugPrint(3, "data has no comma");
 		nzFree(copy);
-		return 0;
+		return false;
 	}
 	*comma++ = 0;
 	unpercentString(comma);
@@ -2353,6 +2386,7 @@ static bool dataConnect(struct i_get *g)
 	debugPrint(4, "content length %lld", g->hcl);
 	nzFree(copy);
 	g->cfn = cloneString("data:");
+	g->cfn = suggestedDownloadName(g->gotag);
 	return true;
 }
 
@@ -2750,7 +2784,9 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 
 // There's a special case where this function is used
 // by the imap client to see if the server is move capable.
-	if (ismc & isimap && info_desc == CURLINFO_HEADER_IN &&
+// Unfortunately this check runns all the while we are at db4, even for http etc,
+// but how often are we at db4?
+	if (info_desc == CURLINFO_HEADER_IN &&
 	    size > 17 && !strncmp(data, "* CAPABILITY IMAP", 17)) {
 		char *s;
 // data may not be null terminated; can't use strstr
@@ -2790,7 +2826,15 @@ static void setup_download(struct i_get *g)
 {
 	const char *filepart;
 	const char *answer;
-	char *fp2, *s;
+	char *fp2 = 0, *s;
+	bool preset = false;
+
+// did we preset a filename?
+	if((answer = down_prefile)) {
+		preset = true;
+		if(stringEqual(answer, "%")) goto default_filename;
+		goto got_answer;
+	}
 
 /* if not run from a terminal then just return. */
 	if (!isInteractive) {
@@ -2798,6 +2842,7 @@ static void setup_download(struct i_get *g)
 		return;
 	}
 
+default_filename:
 	if (g->cdfn)
 		filepart = g->cdfn;
 	else
@@ -2807,33 +2852,46 @@ static void setup_download(struct i_get *g)
 	for (s = fp2; *s; ++s)
 		if (*s == '/' || *s == '\\')
 			*s = '_';
+	if(preset) { answer = fp2; goto got_answer; }
 
 top:
 	answer = getFileName(g->down_msg, fp2, false, true);
+got_answer:
 /* space for a filename means read into memory */
 	if (stringEqual(answer, " ")) {
 		g->down_state = 0;	// in memory download
-		nzFree(fp2);
+		nzFree(fp2), nzFree(down_prefile), down_prefile = 0;
 		return;
 	}
 
 	if (stringEqual(answer, "x") || stringEqual(answer, "X")) {
 		g->down_state = -1;
 		setError(MSG_DownAbort);
-		nzFree(fp2);
+		nzFree(fp2), nzFree(down_prefile), down_prefile = 0;
 		return;
 	}
 
 	if (!envFileDown(answer, &answer)) {
-		showError();
-		goto top;
+		if(!preset) {
+			showError();
+			goto top;
+		}
+		nzFree(fp2), nzFree(down_prefile), down_prefile = 0;
+		g->down_state = -1;
+		return;
 	}
 
 	g->down_fd = creat(answer, MODE_rw);
 	if (g->down_fd < 0) {
-		i_printf(MSG_NoCreate2, answer);
-		nl();
-		goto top;
+		if(!preset) {
+			i_printf(MSG_NoCreate2, answer);
+			nl();
+			goto top;
+		}
+		setError(MSG_NoCreate2, answer);
+		nzFree(fp2), nzFree(down_prefile), down_prefile = 0;
+		g->down_state = -1;
+		return;
 	}
 
 	nzFree(fp2);
@@ -2850,6 +2908,7 @@ top:
 	}
 
 	g->down_state = (down_bg ? 5 : 2);
+	nzFree(down_prefile), down_prefile = 0;
 }
 
 /* show background jobs and return the number of jobs pending */
