@@ -11,15 +11,17 @@
 Window *cw;
 Frame *cf;
 int gfsn; // global frame sequence number
+int ignore;
 const char *progname;
 const char eol[] = "\r\n";
-const char *version = "3.8.6";
+const char *version = "3.8.9";
 char *changeFileName;
 char *configFile, *addressFile, *cookieFile, *emojiFile;
 char *mailDir, *mailUnread, *mailStash, *mailReply;
 char *sigFile, *sigFileEnd;
 char *cacheDir;
 int cacheSize = 1000, cacheCount = 10000;
+bool hlocal; // http local, from cache only
 char *ebTempDir, *ebUserDir;
 char *userAgents[MAXAGENT + 1];
 char *currentAgent;
@@ -43,7 +45,7 @@ static struct DBTABLE dbtables[MAXDBT];
 static int numTables;
 volatile bool intFlag;
 time_t intStart;
-bool ismc, isimap, passMail;
+bool ismc, passMail, attimg;
 // next two variables work around curl bug 7284
 bool inInput, inInitFunction, listNA;
 char *dbarea, *dblogin, *dbpw;	// to log into the database
@@ -373,7 +375,7 @@ const char *mailRedirect(const char *to, const char *from,
 	int rlen = strlen(reply);
 	int slen = strlen(subj);
 	int tlen = strlen(to);
-	int mlen;		// length of match string
+	int mlen = 0;		// length of match string
 	unsigned i;
 	int k;
 	struct ebhost *f = ebhosts;
@@ -486,9 +488,6 @@ int main(int argc, char **argv)
 	selectLanguage();
 	setHTTPLanguage(eb_language);
 
-// bring HISTCONTROL down to edbrowse
-	setHistcontrol();
-
 // Establish the home directory, and standard edbrowse files thereunder.
 	home = getenv("HOME");
 
@@ -506,7 +505,7 @@ int main(int argc, char **argv)
 	if (fileTypeByName(configFile, 0) == 0) {
 		int fh = creat(configFile, MODE_private);
 		if (fh >= 0) {
-			write(fh, ebrc_string, strlen(ebrc_string));
+			ignore = write(fh, ebrc_string, strlen(ebrc_string));
 			close(fh);
 			i_printfExit(MSG_Personalize, configFile);
 		}
@@ -603,9 +602,7 @@ int main(int argc, char **argv)
 			if (!maxAccount)
 				i_printfExit(MSG_NoMailAcc);
 			if (*s == 'f') {
-				account = 0;
-				dofetch = true;
-				++s;
+				account = 0, dofetch = true, ++s;
 				if (*s == 'm')
 					domail = true, ++s;
 			} else {
@@ -631,6 +628,7 @@ int main(int argc, char **argv)
 	}			// options
 
 	srand(time(0));
+	loadAddressBook();
 
 	if (ismc) {
 		char **reclist, **atlist;
@@ -643,24 +641,20 @@ int main(int argc, char **argv)
 			if (dofetch) {
 				int nfetch = 0;
 				if (account) {
-					isimap = accounts[account - 1].imap;
-					if (isimap)
-						domail = false;
+					if (accounts[account - 1].imap) domail = false;
 					nfetch = fetchMail(account);
+// fetchMail does not return in imap mode.
 				} else {
 					nfetch = fetchAllMail();
 				}
 				if (!domail) {
-					if (nfetch)
-						i_printf(MSG_MessagesX, nfetch);
-					else
-						i_puts(MSG_NoMail);
+					if (nfetch) i_printf(MSG_MessagesX, nfetch);
+					else i_puts(MSG_NoMail);
 				}
 			}
 
-			if (domail) {
-				scanMail();
-			}
+			if (domail)
+				scanUnreadMail();
 
 			exit(0);
 		}
@@ -708,6 +702,10 @@ int main(int argc, char **argv)
 
 	signal(SIGINT, catchSig);
 
+// a child process terminates, either from a plugin or from a w !command,
+// all of edbrowse should not crash.
+	signal(SIGPIPE, SIG_IGN);
+
 	js_main();
 
 // This sanity check on number of files assumes they are all files,
@@ -857,8 +855,9 @@ int main(int argc, char **argv)
 static void *inputForever(void *ptr)
 {
 	foreground_thread = pthread_self();
+	printPrompt();
 	while (true) {
-		pst p = inputLine();
+		pst p = inputLine(false);
 		pst save_p = clonePstring(p);
 		if (perl2c((char *)p)) {
 			i_puts(MSG_EnterNull);
@@ -906,6 +905,104 @@ static const char *ip_ptr;
 static const char **endl_ptr;
 static const char **args_ptr;
 static int *argl_ptr;
+static int *rb_ln, *rb_ln2, *rb_b;
+
+struct ebSettings {
+	bool rl, endm, lna, H, ci, sg, su8, sw, ebre, bd, iu, hf, hr, vs, hlocal, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse, fllo, dno, ebvar, flow, attimg;
+	uchar dw, ls_sort;
+	char lsformat[12], showProgress;
+	char *currentAgent;
+	int agentIndex, context, cx_previous, debugLevel, timerspeed;
+	int ll, fll;
+};
+
+static void saveEbSettings(struct ebSettings *s)
+{
+	s->currentAgent = currentAgent, s->agentIndex = agentIndex;
+	s->context = context;
+	s->cx_previous = cx_previous, cx_previous = 0;
+	s->debugLevel = debugLevel;
+	s->timerspeed = timerspeed;
+	s->dw = dirWrite;
+	s->dno = dno;
+	s->ebvar = ebvar;
+	s->flow = flow;
+	s->attimg = attimg;
+	s->ls_sort = ls_sort;
+	strcpy(s->lsformat, lsformat);
+	s->showProgress = showProgress;
+	s->bg = down_bg;
+	s->jsbg = down_jsbg;
+	s->iu = iuConvert;
+	s->bd = binaryDetect;
+	s->rl = inputReadLine;
+	s->can = curlAuthNegotiate;
+	s->lna = listNA;
+	s->fbc = fetchBlobColumns;
+	s->ls_reverse = ls_reverse;
+	s->sw = searchWrap;
+	s->ebre = ebre;
+	s->hr = allowRedirection;
+	s->sr = sendReferrer;
+	s->js = allowJS;
+	s->ftpa = ftpActive;
+	s->H = helpMessagesOn;
+	s->pg = pluginsOn;
+	s->hf = showHiddenFiles;
+	s->showall = showall;
+	s->endm = endMarks;
+	s->sg = searchStringsAll;
+	s->ci = caseInsensitive;
+	s->su8 = re_utf8;
+	s->vs = verifyCertificates;
+	s->hlocal = hlocal;
+	s->ll = displayLength;
+	s->fll = formatLineLength;
+	s->fllo = formatOverflow;
+}
+
+static void restoreEbSettings(struct ebSettings *s)
+{
+	currentAgent = s->currentAgent, agentIndex = s->agentIndex;
+	cx_previous = (context == s->context) ? s->cx_previous : s->context;
+	debugLevel = s->debugLevel;
+	timerspeed = s->timerspeed;
+	dirWrite = s->dw;
+	dno = s->dno;
+	ebvar = s->ebvar;
+	flow = s->flow;
+	attimg = s->attimg;
+	ls_sort = s->ls_sort;
+	strcpy(lsformat, s->lsformat);
+	showProgress = s->showProgress;
+	down_bg = s->bg;
+	down_jsbg = s->jsbg;
+	iuConvert = s->iu;
+	binaryDetect = s->bd;
+	inputReadLine = s->rl;
+	curlAuthNegotiate = s->can;
+	listNA = s->lna;
+	ls_reverse = s->ls_reverse;
+	searchWrap = s->sw;
+	ebre = s->ebre;
+	allowRedirection = s->hr;
+	sendReferrer = s->sr;
+	allowJS = s->js;
+	ftpActive = s->ftpa;
+	helpMessagesOn = s->H;
+	pluginsOn = s->pg;
+	showHiddenFiles = s->hf;
+	showall = s->showall;
+	endMarks = s->endm;
+	searchStringsAll = s->sg;
+	caseInsensitive = s->ci;
+	re_utf8 = s->su8;
+	verifyCertificates = s->vs;
+	hlocal = s->hlocal;
+	displayLength = s->ll;
+	formatLineLength = s->fll;
+	formatOverflow = s->fllo;
+}
 
 static char *substituteArgs(void)
 {
@@ -960,16 +1057,8 @@ int runEbFunction(const char *line)
 	int rc = -1;
 	char stack[MAXNEST];
 	int loopcnt[MAXNEST];
-
 // local copies of settings, to restore after function runs.
-	struct {
-		bool rl, endm, lna, H, ci, sg, su8, sw, ebre, bd, iu, hf, hr, vs, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse, fllo, dno, ebvar, flow;
-		uchar dw, ls_sort;
-		char lsformat[12], showProgress;
-		char *currentAgent;
-		int agentIndex, context, cx_previous, debugLevel, timerspeed;
-		int ll, fll;
-	} save;
+	struct ebSettings save;
 
 /* Separate function name and arguments */
 	spaceCrunch(linecopy, true, false);
@@ -1006,45 +1095,7 @@ int runEbFunction(const char *line)
 
 	if(!stringEqual(linecopy, "init") && strncmp(linecopy, "set", 3)) {
 		restore = true;
-		save.currentAgent = currentAgent, save.agentIndex = agentIndex;
-		save.context = context;
-		save.cx_previous = cx_previous, cx_previous = 0;
-		save.debugLevel = debugLevel;
-		save.timerspeed = timerspeed;
-		save.dw = dirWrite;
-		save.dno = dno;
-		save.ebvar = ebvar;
-		save.flow = flow;
-		save.ls_sort = ls_sort;
-		strcpy(save.lsformat, lsformat);
-		save.showProgress = showProgress;
-		save.bg = down_bg;
-		save.jsbg = down_jsbg;
-		save.iu = iuConvert;
-		save.bd = binaryDetect;
-		save.rl = inputReadLine;
-		save.can = curlAuthNegotiate;
-		save.lna = listNA;
-		save.fbc = fetchBlobColumns;
-		save.ls_reverse = ls_reverse;
-		save.sw = searchWrap;
-		save.ebre = ebre;
-		save.hr = allowRedirection;
-		save.sr = sendReferrer;
-		save.js = allowJS;
-		save.ftpa = ftpActive;
-		save.H = helpMessagesOn;
-		save.pg = pluginsOn;
-		save.hf = showHiddenFiles;
-		save.showall = showall;
-		save.endm = endMarks;
-		save.sg = searchStringsAll;
-		save.ci = caseInsensitive;
-		save.su8 = re_utf8;
-		save.vs = verifyCertificates;
-		save.ll = displayLength;
-		save.fll = formatLineLength;
-		save.fllo = formatOverflow;
+		saveEbSettings(&save);
 	}
 
 /* collect arguments, ~0 first */
@@ -1145,7 +1196,7 @@ ahead:
 			goto soft_fail;
 
 		ip_ptr = ip;
-		endl_ptr = &endl;
+		endl_ptr = &endl, rb_ln = 0;
 		args_ptr = args;
 		argl_ptr = argl;
 		new = substituteArgs();
@@ -1176,45 +1227,8 @@ done:
 	nzFree(linecopy);
 	nzFree(fncopy);
 	nzFree(allargs);
-	if(restore) {
-		currentAgent = save.currentAgent, agentIndex = save.agentIndex;
-		cx_previous = (context == save.context) ? save.cx_previous : save.context;
-		debugLevel = save.debugLevel;
-		timerspeed = save.timerspeed;
-		dirWrite = save.dw;
-		dno = save.dno;
-		ebvar = save.ebvar;
-		flow = save.flow;
-		ls_sort = save.ls_sort;
-		strcpy(lsformat, save.lsformat);
-		showProgress = save.showProgress;
-		down_bg = save.bg;
-		down_jsbg = save.jsbg;
-		iuConvert = save.iu;
-		binaryDetect = save.bd;
-		inputReadLine = save.rl;
-		curlAuthNegotiate = save.can;
-		listNA = save.lna;
-		ls_reverse = save.ls_reverse;
-		searchWrap = save.sw;
-		ebre = save.ebre;
-		allowRedirection = save.hr;
-		sendReferrer = save.sr;
-		allowJS = save.js;
-		ftpActive = save.ftpa;
-		helpMessagesOn = save.H;
-		pluginsOn = save.pg;
-		showHiddenFiles = save.hf;
-		showall = save.showall;
-		endMarks = save.endm;
-		searchStringsAll = save.sg;
-		caseInsensitive = save.ci;
-		re_utf8 = save.su8;
-		verifyCertificates = save.vs;
-		displayLength = save.ll;
-		formatLineLength = save.fll;
-		formatOverflow = save.fllo;
-	}
+	if(restore)
+		restoreEbSettings(&save);
 	return rc;
 
 soft_fail:
@@ -1225,9 +1239,22 @@ soft_fail:
 // so we can append text from within a script, like a here document
 const char *getInputLineFromScript(void)
 {
-	const char *endl = *endl_ptr;
-	const char *ip = endl + 1;
 	char *new;
+	const char *endl, *ip;
+
+// exactly one of endl_ptr and rb_ln is not null.
+	if(rb_ln) {
+		int b = *rb_b, ln = *rb_ln, ln2 = *rb_ln2;
+		const Window *w = sessionList[b].lw;
+		int dol = w->dol;
+// the buffer could run out before . terminates the input
+		if(ln == dol || ln == ln2) return ".\n";
+		*rb_ln = ++ln;
+		return (const char *)fetchLineContext(ln, 0, b);
+	}
+
+	endl = *endl_ptr;
+	ip = endl + 1;
 // the script could run out before . terminates the input
 	if(!*ip) return ".\n";
 	endl = strchr(ip, '\n');
@@ -1246,6 +1273,53 @@ const char *getInputLineFromScript(void)
 	return new;
 }
 
+bool runBuffer(int b, const Window *w, bool stopflag, int ln1, int ln2)
+{
+	int ln;
+	bool ok;
+	char b0[16];
+	sprintf(b0, "session %d", b);
+	if(!ln1) {
+// did not run through atPartCracker()
+		if(!b) { setError(MSG_Session0); return false; }
+		if(!cxActive(b, true)) return false;
+		if(b == context) {
+			setError(MSG_SessionCurrent, context);
+			return false;
+		}
+		w = sessionList[b].lw;
+		ln1 = 1, ln2 = w->dol;
+	}
+	if(w->dirMode | w->binMode | w->sqlMode | w->irciMode | w->ircoMode | w->browseMode) {
+		setError(MSG_SessionMode, b);
+		return false;
+	}
+
+// loop over lines
+	for(ln = ln1; ln <= ln2; ++ln) {
+// has the window gone away?
+// Did you do something perverse like q2 while running commands in buffer 2?
+// There's almost no way to check for this safely, but here is a modest attempt.
+		if(!w->map) break;
+		if(ln > w->dol) break;
+		char *p = (char*)fetchLineWindow(ln, 0, w);
+		char *s = strchr(p, '\n');
+		if(!s) { // line contains nulls
+			setError(MSG_EBRC_Nulls, b0, ln);
+			free(p);
+			return false;
+		}
+		*s = 0;
+		rb_ln = &ln, rb_ln2 = &ln2, rb_b = &b, endl_ptr = 0;
+// Here we go!
+		debugPrint(3, "< %s", p);
+		ok = edbrowseCommand(p, true);
+		free(p);
+		if(stopflag && !ok) return false;
+	}
+	return true;
+}
+
 struct DBTABLE *findTableDescriptor(const char *sn)
 {
 	int i;
@@ -1304,6 +1378,7 @@ void unreadConfigFile(void)
 
 	nzFree(addressFile), addressFile = 0;
 	nzFree(emojiFile), emojiFile = 0, clearEmojis();
+	nzFree(irclog), irclog = 0;
 	nzFree(cookieFile), cookieFile = 0;
 	nzFree(sslCerts), sslCerts = 0;
 	nzFree(pubKey), pubKey = 0;
@@ -1320,14 +1395,15 @@ void unreadConfigFile(void)
 
 /* Order is important here: mail{}, mime{}, table{}, then global keywords */
 #define MAILWORDS 0
-#define MIMEWORDS 12
-#define TABLEWORDS 20
-#define GLOBALWORDS 24
+#define MIMEWORDS 15
+#define TABLEWORDS 23
+#define GLOBALWORDS 27
 
 static const char *const keywords[] = {
 	"inserver", "outserver", "login", "password", "from", "reply",
 	"inport", "outport",
 	"to", "cc", "bcc", "attach",
+	"imask", "isub", "dx",
 	"type", "desc", "suffix", "protocol", "program",
 	"content", "outtype", "urlmatch",
 	"tname", "tshort", "cols", "keycol",
@@ -1336,7 +1412,7 @@ static const char *const keywords[] = {
 	"webtimer", "mailtimer", "certfile", "datasource", "proxy",
 	"agentsite", "localizeweb", "imapfetch", "novs", "cachesize",
 	"adbook", "envelope", "emojis", "emoji",
-"include", "js", "pubkey", 0};
+"include", "js", "pubkey", "irclog", 0};
 
 /* Read the config file and populate the corresponding data structures. */
 /* This routine succeeds, or aborts via one of these macros. */
@@ -1395,8 +1471,7 @@ static bool preConfigFile(char *buf, int buflen)
 				v[1] = 'm';
 				t = v + 2;
 			}
-			if (stringEqual(last, "plugin{") ||
-			    stringEqual(last, "mime{")) {
+			if (stringEqual(last, "plugin{")) {
 				*v = '\x81';
 				v[1] = 'e';
 				t = v + 2;
@@ -1468,10 +1543,8 @@ static bool preConfigFile(char *buf, int buflen)
 				q = last + 9;
 				if (*q == 0 || *q == '{' || *q == '(')
 					cfgLine0(MSG_EBRC_NoFnName);
-#if 0
 				if (isdigitByte(*q))
 					cfgLine0(MSG_EBRC_FnDigit);
-#endif
 				while (isalnumByte(*q))
 					++q;
 				if (q - last - 9 > 10)
@@ -1517,9 +1590,9 @@ bool readConfigFile(void)
 	int nest, ln, j;
 	int sn = 0;		/* script number */
 	char stack[MAXNEST];
-	struct MACCOUNT *act;
-	struct MIMETYPE *mt;
-	struct DBTABLE *td;
+	struct MACCOUNT *act = 0;
+	struct MIMETYPE *mt = 0;
+	struct DBTABLE *td = 0;
 	struct cfgFile *f, *g;
 
 	unreadConfigFile();
@@ -1635,37 +1708,37 @@ inside:
 			cfgLine1(MSG_EBRC_NoAttr, s);
 
 		switch (n) {
-		case 0:	/* inserver */
+		case 0:	// inserver
 			act->inurl = v;
 			continue;
 
-		case 1:	/* outserver */
+		case 1:	// outserver
 			act->outurl = v;
 			continue;
 
-		case 2:	/* login */
+		case 2:	// login
 			act->login = v;
 			continue;
 
-		case 3:	/* password */
+		case 3:	// password
 			act->password = v;
 			continue;
 
-		case 4:	/* from */
+		case 4:	// from
 			act->from = v;
 			continue;
 
-		case 5:	/* reply */
+		case 5:	// reply
 			act->reply = v;
 			continue;
 
-		case 6:	/* inport */
+		case 6:	// inport
 			if (*v == '*')
 				act->inssl = 1, ++v;
 			act->inport = atoi(v);
 			continue;
 
-		case 7:	/* outport */
+		case 7:	// outport
 			if (*v == '+')
 				act->outssl = 4, ++v;
 			if (*v == '^')
@@ -1688,63 +1761,76 @@ inside:
 			}
 			continue;
 
-		case 12:	/* type */
+		case 12:	// imask
+			act->maskon = act->maskactive = true;
+			act->masktext = v;
+			continue;
+
+		case 13:	// isub
+			act->isub = v;
+			continue;
+
+		case 14:	// dx
+			act->dxon = true;
+			act->dxtext = v;
+			continue;
+
+		case 15:	// type
 			mt->type = v;
 			continue;
 
-		case 13:	/* desc */
+		case 16:	// desc
 			mt->desc = v;
 			continue;
 
-		case 14:	/* suffix */
+		case 17:	// suffix
 			mt->suffix = v;
 			continue;
 
-		case 15:	/* protocol */
+		case 18:	// protocol
 			mt->prot = v;
 			continue;
 
-		case 16:	/* program */
+		case 19:	// program
 			mt->program = v;
 			continue;
 
-		case 17:	/* content */
+		case 20:	// content
 			mt->content = v;
 			continue;
 
-		case 18:	/* outtype */
+		case 21:	// outtype
 			c = tolower(*v);
 			if (c != 'h' && c != 't')
 				cfgLine0(MSG_EBRC_Outtype);
 			mt->outtype = c;
 			continue;
 
-		case 19:	/* urlmatch */
+		case 22:	// urlmatch
 			mt->urlmatch = v;
 			continue;
 
-		case 20:	/* tname */
+		case 23:	// tname
 			td->name = v;
 			continue;
 
-		case 21:	/* tshort */
+		case 24:	// tshort
 			td->shortname = v;
 			continue;
 
-		case 22:	/* cols */
+		case 25:	// cols
 			while (*v) {
 				if (td->ncols == MAXTCOLS)
 					cfgLine1(MSG_EBRC_ManyCols, MAXTCOLS);
 				td->cols[td->ncols++] = v;
 				q = strchr(v, ',');
-				if (!q)
-					break;
+				if (!q) break;
 				*q = 0;
 				v = q + 1;
 			}
 			continue;
 
-		case 23:	/* keycol */
+		case 26:	// keycol
 			if (!isdigitByte(*v))
 				cfgLine0(MSG_EBRC_KeyNotNb);
 			td->key1 = (uchar) strtol(v, &v, 10);
@@ -1754,7 +1840,7 @@ inside:
 				cfgLine1(MSG_EBRC_KeyOutRange, td->ncols);
 			continue;
 
-		case 24:	/* downdir */
+		case 27:	// downdir
 			nzFree(downDir), downDir = 0; // in case called more than once
 			v = envFileAlloc(v);
 			if(!v) continue;
@@ -1764,7 +1850,7 @@ inside:
 			downDir = v;
 			continue;
 
-		case 25:	/* maildir */
+		case 28:	// maildir
 			nzFree(mailDir), mailDir = 0;
 			nzFree(mailUnread), mailUnread = 0;
 			nzFree(mailReply), mailReply = 0;
@@ -1785,7 +1871,7 @@ inside:
 			sprintf(mailReply, "%s/.reply", v);
 			continue;
 
-		case 26:	/* agent */
+		case 29:	// agent
 			for (j = 0; j < MAXAGENT; ++j)
 				if (!userAgents[j])
 					break;
@@ -1794,7 +1880,7 @@ inside:
 			userAgents[j] = v;
 			continue;
 
-		case 27:	/* jar */
+		case 30:	// jar
 			nzFree(cookieFile), cookieFile = 0;
 			v = envFileAlloc(v);
 			if(!v) continue;
@@ -1809,7 +1895,7 @@ inside:
 			cookieFile = v;
 			continue;
 
-		case 28:	/* nojs */
+		case 31:	// nojs
 			if (*v == '.')
 				++v;
 /* Is this essential?
@@ -1820,20 +1906,20 @@ inside:
 			add_ebhost(v, 'j');
 			continue;
 
-		case 29:	/* cachedir */
+		case 32:	// cachedir
 			nzFree(cacheDir);
 			cacheDir = cloneString(v);
 			continue;
 
-		case 30:	/* webtimer */
+		case 33:	// webtimer
 			webTimeout = atoi(v);
 			continue;
 
-		case 31:	/* mailtimer */
+		case 34:	// mailtimer
 			mailTimeout = atoi(v);
 			continue;
 
-		case 32:	/* certfile */
+		case 35:	// certfile
 			nzFree(sslCerts), sslCerts = 0;
 			v = envFileAlloc(v);
 			if(!v) continue;
@@ -1847,15 +1933,15 @@ inside:
 			sslCerts = v;
 			continue;
 
-		case 33:	/* datasource */
+		case 36:	// datasource
 			setDataSource(v);
 			continue;
 
-		case 34:	/* proxy */
+		case 37:	// proxy
 			add_proxy(v);
 			continue;
 
-		case 35:	// agentsite
+		case 38:	// agentsite
 			spaceCrunch(v, true, true);
 			q = strchr(v, ' ');
 			if (!q || strchr(q + 1, ' ') ||
@@ -1870,21 +1956,17 @@ inside:
 			ebhosts[ebhosts_avail - 1].n = j;
 			continue;
 
-		case 36:	/* localizeweb */
+		case 39:	// localizeweb
 /* We should probably allow autodetection of language. */
 /* E.G., the keyword auto indicates that you want autodetection. */
 			setHTTPLanguage(v);
 			continue;
 
-		case 37:	/* imap fetch limit */
-			imapfetch = atoi(v);
-			if (imapfetch < 5) imapfetch = 5;
-// there isn't an upper limit when set interactively, so idk
-			if (imapfetch >= 1000)
-				imapfetch = 1000;
+		case 40:	// imap fetch limit
+			if(stringIsNum(v) >= 0) setFetchLimit(v);
 			continue;
 
-		case 38:	/* novs */
+		case 41:	// novs
 			if (*v == '.')
 				++v;
 			q = strchr(v, '.');
@@ -1893,7 +1975,7 @@ inside:
 			add_ebhost(v, 'v');
 			continue;
 
-		case 39:	/* cachesize */
+		case 42:	// cachesize
 			cacheSize = atoi(v);
 			if (cacheSize <= 0)
 				cacheSize = 0;
@@ -1901,7 +1983,7 @@ inside:
 				cacheSize = 10000;
 			continue;
 
-		case 40:	// adbook
+		case 43:	// adbook
 			nzFree(addressFile), addressFile = 0;
 			v = envFileAlloc(v);
 			if(!v) continue;
@@ -1911,11 +1993,11 @@ inside:
 			addressFile = v;
 			continue;
 
-		case 41:	// envelope
+		case 44:	// envelope
 			setEnvelopeFormat(v);
 			continue;
 
-		case 42: case 43:	// emojis
+		case 45: case 46:	// emojis
 			nzFree(emojiFile), emojiFile = 0;
 			v = envFileAlloc(v);
 			if(!v) continue;
@@ -1926,7 +2008,7 @@ inside:
 			loadEmojis();
 			continue;
 
-		case 44: // include
+		case 47: // include
 			v = envFileAlloc(v);
 			if(!v) continue;
 			if(!fileIntoMemory(v, &incbuf, &inclen, 0)) {
@@ -1953,7 +2035,7 @@ inside:
 			f = g;
 			goto top;
 
-		case 45:	/* js */
+		case 48:	// js
 			if (*v == '.')
 				++v;
 /* Is this essential?
@@ -1964,10 +2046,12 @@ inside:
 			add_ebhost(v, 'J');
 			continue;
 
-		case 46:	/* pubkey */
+		case 49:	// pubkey
+// this feature is undocumented
 			nzFree(pubKey), pubKey = 0;
 			v = envFileAlloc(v);
 			if(!v) continue;
+// error legs do not free v, who cares.
 			ftype = fileTypeByName(v, 0);
 			if (ftype && ftype != 'f')
 				cfgAbort1(MSG_EBRC_KeyNoFile, v);
@@ -1978,6 +2062,11 @@ inside:
 			pubKey = v;
 			continue;
 
+		case 50:	// irclog
+			nzFree(irclog), irclog = 0;
+			irclog = envFileAlloc(v);
+			continue;
+
 		default:
 			cfgLine1(MSG_EBRC_KeywordNYI, s);
 		}		/* switch */
