@@ -169,9 +169,9 @@ enum {
 	MSG_FormReset,
 	MSG_FormSubmit,
 	MSG_EBRC_Outtype,
-	MSG_Many,
+	MSG_Multiple,
 	MSG_Recommended,
-	MSG_Close,
+	MSG_ImapUsage,
 	MSG_NoOptionsMatch,
 	MSG_MailSending,
 	MSG_MailSent,
@@ -306,7 +306,7 @@ enum {
 	MSG_QAfter,
 	MSG_DirRename,
 	MSG_TableRename,
-	MSG_BufferAppend,
+	MSG_NotImap,
 	MSG_NoFileSpecified,
 	MSG_NoDirWrite,
 	MSG_NoDBWrite,
@@ -695,7 +695,7 @@ enum {
 	MSG_NullCharInList,
 	MSG_NullPtr,
 	MSG_IoctlError,
-	MSG_notused598,
+	MSG_WBang,
 	MSG_CreateFail,
 	MSG_InvalidFopen,
 	MSG_BadDirSlash,
@@ -816,4 +816,39 @@ enum {
 	MSG_GetFrag2,
 	MSG_UpdateEBVarOff,
 	MSG_UpdateEBVarOn,
+	MSG_CmdFail,
+	MSG_SameFolder,
+	MSG_HttpInternet,
+	MSG_HttpLocal,
+	MSG_SessionMode,
+	MSG_NoAuthor,
+	MSG_NoGenerator,
+	MSG_BWAfter,
+MSG_ImaskOff,
+MSG_ImaskOn,
+	MSG_NoMask,
+	MSG_TextRecBuf,
+	MSG_PartRead,
+	MSG_PartBoundary,
+	MSG_ImapCompat,
+	MSG_ImapCommand,
+	MSG_ImapCommandS,
+	MSG_MailImap,
+	MSG_ReImap,
+	MSG_AudioImap,
+	MSG_BreakImap,
+	MSG_ImapRename,
+	MSG_ImapConfig,
+	MSG_NMC,
+	MSG_ImapCompat0,
+	MSG_AttImgOff,
+	MSG_AttImgOn,
+	MSG_ShowChanOff,
+	MSG_ShowChanOn,
+	MSG_NotIrcSend,
+	MSG_DotMoved,
+	MSG_NoCreate3,
+	MSG_NoDelete3,
+	MSG_NoRename3,
+	MSG_FolderNameBad,
 };
