@@ -12,7 +12,7 @@
 #define ABDELIMITER ':'		// character between fields in address book entry
 #define FULLNAMEDELIMITER '>'		// character between email address and full name in address book entry
 #define REPLACEMENTFULLNAMEDELIMITER '\n' // character temporarily used between email address and full name
-#define MAXRECAT 100		// max number of recipients or attachments
+#define MAXRECAT 1000		// max number of recipients or attachments
 #define MAXMSLINE 1024		// max mail server line
 #define LONGLINELIMIT 76
 #define LONGWORDLIMIT 998
@@ -271,18 +271,17 @@ static char *qpEncode(const char *line)
 	int l;
 	const char *s;
 	char c;
-
 	newbuf = initString(&l);
 	for (s = line; (c = *s); ++s) {
-		if ((c < '\n' && c != '\t') || c == '=') {
+		if ((signed char)c < '\n' || c == '=' || c == '_' || c == '?') {
 			char expand[4];
 			sprintf(expand, "=%02X", (uchar) c);
 			stringAndString(&newbuf, &l, expand);
 		} else {
+			if(c == ' ') c = '_';
 			stringAndChar(&newbuf, &l, c);
 		}
 	}
-
 	return newbuf;
 }
 
@@ -323,7 +322,7 @@ static char *isoEncode(char *start, char *end)
 package:
 	len = strlen(s);
 	t = allocMem(len + 20);
-	sprintf(t, "=?ISO-8859-1?%c?%s?=", code, s);
+	sprintf(t, "=?UTF-8?%c?%s?=", code, s);
 	nzFree(s);
 	return t;
 }
@@ -363,7 +362,7 @@ bool *long_p)
 {
 	char *buf;
 	char c;
-	bool longline, longword, cr, endlinespace, flowed;
+	bool longline, longword, cr, endlinespace, startlinedot, flowed;
 	char *s, *t, *v;
 	char *ct, *ce;		// content type, content encoding
 	int buflen, i, cx;
@@ -491,7 +490,7 @@ empty:
 						setError(MSG_SigAccess);
 						goto freefail;
 					}
-					read(fd, buf + buflen, n);
+					ignore = read(fd, buf + buflen, n);
 					close(fd);
 					buflen += n;
 					buf[buflen] = 0;
@@ -538,13 +537,15 @@ empty:
 
 /* Count the nonascii characters */
 	nacount = nullcount = linelength = wordlength = 0;
-	longline = longword = cr = endlinespace = flowed = false;
+	longline = longword = cr = endlinespace = startlinedot = flowed = false;
 	for (t = buf, i = 0; i < buflen; ++i, ++t) {
 		c = *t;
 		if (c == '\0')
 			++nullcount;
 		if (c & 0x80)
 			++nacount;
+		if (c == '.' && linelength == 0 && t != buf)
+			startlinedot = true;
 		if (c == '\n') {
 			if(linelength > LONGLINELIMIT) longline = true;
 			linelength = 0;
@@ -597,7 +598,7 @@ empty:
 	}
 
 	ce = (nacount ? "8bit" : "7bit");
-	if (webform || !(longline || (ismail && flow && endlinespace))) {
+	if (webform || !(longline || (ismail && flow && endlinespace) || (!ismail && startlinedot))) {
 		buf[buflen] = 0;
 		goto success;
 	}
@@ -608,7 +609,7 @@ empty:
 	char *newbuf;
 	int l, colno = 0, space = 0;
 	if (ismail && flow && !longword) flowed = true;
-	else ce = "quoted-printable";
+	else if (longline) ce = "quoted-printable";
 	newbuf = initString(&l);
 	v = buf + buflen;
 	for (s = buf; s < v; ++s) {
@@ -635,7 +636,7 @@ empty:
 			}
 		}
 // do we have to =expand this character?
-		if (!flowed &&
+		if (!flowed && longline &&
 		((((uchar)c < '\n' && c != '\t') ||
 		    c == '=' ||
 		    (uchar)c == 0xff ||
@@ -645,6 +646,10 @@ empty:
 			sprintf(expand, "=%02X", (uchar) c);
 			stringAndString(&newbuf, &l, expand);
 			colno += 3;
+		} else if (!ismail && startlinedot && c == '.' && s != buf && s[-1] == '\n') {
+// In an attachment, add an extra dot at the start of a line (other than the first line).
+			stringAndString(&newbuf, &l, "..");
+			colno += 2;
 		} else {
 			stringAndChar(&newbuf, &l, c);
 // make a utf8 char look like 1
@@ -1327,6 +1332,10 @@ bool sendMailCurrent(int sm_account, bool dosig)
 		setError(MSG_MailIrc);
 		return false;
 	}
+	if (cw->imapMode1 | cw->imapMode2) {
+		setError(MSG_MailImap);
+		return false;
+	}
 	if (cw->dirMode) {
 		setError(MSG_MailDir);
 		return false;
