@@ -4,10 +4,10 @@
 
 #include <dirent.h>
 #include <termios.h>
-#include <unistd.h>
 #include <glob.h>
 #include <pwd.h>
 #include <grp.h>
+#include <utime.h>
 
 char emptyString[] = "";
 bool showHiddenFiles, isInteractive;
@@ -21,7 +21,7 @@ bool gotimers = true;
 bool uvw;
 int timerspeed = 1;
 long long fileSize;
-char *downDir, *home;
+char *downDir, *down_prefile, *home;
 bool endMarks;		// ^ $ on listed lines
 uchar dirWrite;		// directories read write
 bool dno; // directory names only
@@ -864,7 +864,7 @@ long long bufferSizeW(const Window *w, bool browsing)
 long long bufferSize(int cx, bool browsing)
 {
 	const Window *w;
-	if (cx <= 0 || cx >= MAXSESSION || (w = sessionList[cx].lw) == 0) {
+	if (cx <= 0 || cx > maxSession || (w = sessionList[cx].lw) == 0) {
 		setError(MSG_SessionInactive, cx);
 		return -1;
 	}
@@ -1122,7 +1122,8 @@ char *conciseSize(size_t n)
 
 char *conciseTime(time_t t)
 {
-	static char buffer[22];
+	static char buffer[24];
+// longest month is 5 bytes
 	static const char *const englishMonths[] = {
 		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
@@ -1359,6 +1360,7 @@ void ttySetEcho(bool enable_echo)
 int getche(void)
 {
 	char c;
+	if(!isInteractive) return getchar();
 	fflush(stdout);
 	ttyRaw(1, 0, true);
 	read(0, &c, 1);
@@ -1369,6 +1371,7 @@ int getche(void)
 int getch(void)
 {
 	char c;
+	if(!isInteractive) return getchar();
 	fflush(stdout);
 	ttyRaw(1, 0, false);
 	read(0, &c, 1);
@@ -1384,8 +1387,10 @@ char getLetter(const char *s)
 		c = getch();
 		if (strchr(s, c))
 			break;
-		printf("\a\b");
-		fflush(stdout);
+		if(isInteractive) {
+			printf("\a\b");
+			fflush(stdout);
+		}
 	}
 	printf("%c", c);
 	return c;
@@ -1422,7 +1427,7 @@ char *getFileName(int msg, const char *defname, bool isnew, bool ws)
 				return spacename;
 			if (!defname)
 				continue;
-/* make a copy just to be safe */
+// make a copy just to be safe
 			l = strlen(defname);
 			if (l >= ABSPATH)
 				l = ABSPATH - 1;
@@ -1431,7 +1436,8 @@ char *getFileName(int msg, const char *defname, bool isnew, bool ws)
 			p = buf;
 		} else
 			defname = 0;
-		if (isnew && fileTypeByName(p, 0)) {
+// see if file exists
+		if (isnew && !stringEqual(p, "x") && !stringEqual(p, "X") && fileTypeByName(p, 0)) {
 			i_printf(MSG_FileExists, p);
 			defname = 0;
 			continue;
@@ -1464,20 +1470,23 @@ void shellProtect(char *t, const char *s)
 
 // get the directory suffix for a file.
 // This only makes sense in directory mode.
-char *dirSuffixContext(int n, int cx)
+char *dirSuffixWindow(int n, const Window *w)
 {
 	static char suffix[4];
-	Window *lw = sessionList[cx].lw;
-
 	suffix[0] = 0;
-	if (lw->dirMode) {
-		suffix[0] = lw->dmap[DTSIZE*n];
-		suffix[1] = lw->dmap[DTSIZE*n + 1];
+	if (w->dirMode) {
+		suffix[0] = w->dmap[DTSIZE*n];
+		suffix[1] = w->dmap[DTSIZE*n + 1];
 		suffix[2] = 0;
 	}
 	return suffix;
 }
 
+static char *dirSuffixContext(int n, int cx)
+{
+	return dirSuffixWindow(n, sessionList[cx].lw);
+}
+
 char *dirSuffix(int n)
 {
 	return dirSuffixContext(n, context);
@@ -1919,12 +1928,12 @@ unlink:
 					if (*ftype == '/' ||
 					fileSizeByName(path) > 200000000) {
 // let mv do the work
-						if(!qc) {
+						if(!qc || strchr(bin, qc)) {
 							setError(MSG_MetaChar);
 							goto abort;
 						}
-						asprintf(&a, "mv -n %c%s%c",
-						qc, path, qc);
+						asprintf(&a, "mv -n %c%s%c %c%s%c",
+						qc, path, qc, qc, bin, qc);
 						j = system(a);
 						free(a);
 						if(!j) {
@@ -1973,10 +1982,6 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 	}
 
 	if(!relative) {
-				if (dest >= MAXSESSION) {
-					setError(MSG_SessionHigh, dest, MAXSESSION - 1);
-					return false;
-				}
 				if (dest == 0) {
 					setError(MSG_Session0);
 					return false;
@@ -1985,7 +1990,7 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 					setError(MSG_SessionCurrent, dest);
 					return false;
 				}
-				if (!(cw2 = sessionList[dest].lw) || !cw2->dirMode) {
+				if (dest > maxSession || !(cw2 = sessionList[dest].lw) || !cw2->dirMode) {
 					char numstring[12];
 					sprintf(numstring, "%d", dest);
 					setError(MSG_NotDir, numstring);
@@ -2014,8 +2019,8 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 			setError(MSG_NoDown);
 			return false;
 		}
-		dest -= cnt;
-		for(w = cs->lw2; dest; w = w->prev, --dest)  ;
+		cnt -= dest;
+		for(w = cs->lw2; cnt; w = w->prev, --cnt)  ;
 		cw2 = w;
 	}
 
@@ -2049,7 +2054,7 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 		}
 
 		if (!access(path2, 0)) {
-			setError(MSG_DestFileExists);
+			setError(MSG_DestFileExists, path2);
 			free(file);
 			free(path1);
 			return false;
@@ -2106,6 +2111,19 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 					nzFree(rmbuf);
 					return false;
 				}
+
+// reset the time stamp, cause that's what cp does
+				struct stat buf;
+				if (!stat(path1, &buf)) {
+// we can march on if the time isn't adjusted
+#if defined(__APPLE__)
+					struct timespec times[2] = {buf.st_atimespec, buf.st_mtimespec};
+#else
+					struct timespec times[2] = {buf.st_atim, buf.st_mtim};
+#endif
+					utimensat(AT_FDCWD, path2, times, 0);
+				}
+
 				nzFree(rmbuf);
 				if(origcmd == 'm')
 					unlink(path1);
@@ -2122,6 +2140,8 @@ moved:
 		free(path1);
 		if(origcmd == 'm')
 			delText(ln, ln);
+		else
+			cw->dot = ln;
 // add it to the other directory
 		*t++ = '\n';
 		cw = cw2;
@@ -2429,11 +2449,13 @@ int eb_system(const char *cmd, bool print_on_success)
 {
 	int system_ret = system(cmd);
 	if (system_ret != -1) {
-		if (print_on_success && debugLevel > 0)
+		if (print_on_success && debugLevel > 0 && !system_ret)
 			i_puts(MSG_OK);
+		if(system_ret) setError(MSG_CmdFail);
 	} else {
 		i_printf(MSG_SystemCmdFail, system_ret);
 		nl();
+		setError(MSG_CmdFail);
 	}
 	return system_ret;
 }
